@@ -2,8 +2,8 @@ use darling::FromMeta;
 use inflector::cases::snakecase::to_snake_case;
 use proc_macro::TokenStream;
 use proc_macro2::TokenStream as TokenStream2;
-use quote::{format_ident, quote};
-use syn::{parse_macro_input, DeriveInput};
+use quote::{format_ident, quote, quote_spanned};
+use syn::{parse_macro_input, spanned::Spanned, DeriveInput, Type};
 
 use super::attrs::HasManyAttr;
 use super::attrs::{EntityAttr, IndexAttr};
@@ -19,6 +19,24 @@ impl EntityBuilder {
 
     fn build_save_fn(&self, props: &Props) -> TokenStream2 {
         let table_name = props.get_table_name();
+
+        let fields_plain_value_acessors: Vec<TokenStream2> = props
+            .get_fields_plain()
+            .iter()
+            .map(|field| {
+                let name = &field.ident;
+                if let Some(ct) = field.parse_custom_type() {
+                    let ty = ct.ty;
+
+                    let ty_ident = format_ident!("{}", ty);
+
+                    return quote! { &<#ty_ident>::from(&self.#name) };
+                }
+
+                quote! { &self.#name }
+            })
+            .collect();
+
         let fields_plain_names = props.get_fields_plain_names();
 
         let comma_after_default = match fields_plain_names.len() {
@@ -66,7 +84,7 @@ impl EntityBuilder {
                         );
                         let rows = db.query(
                             query,
-                            &[#( &self.#fields_plain_names),*]
+                            &[#( #fields_plain_value_acessors ),*]
                         ).await?;
                         let first_row = rows.first().ok_or(oxidizer::db::Error::Other)?;
                         self.#primary_key_ident = first_row.get::<&str, #primary_key_type>(stringify!(#primary_key_ident));
@@ -85,7 +103,7 @@ impl EntityBuilder {
                         );
                         db.execute(
                             query,
-                            &[#( &self.#fields_plain_names,)* &self.#primary_key_ident],
+                            &[#( #fields_plain_value_acessors,)* &self.#primary_key_ident],
                         ).await?
                     }
                 };
@@ -96,13 +114,39 @@ impl EntityBuilder {
     }
 
     fn build_from_row_fn(&self, props: &Props) -> TokenStream2 {
-        let fields_all_names = props.get_fields_all_names();
-        let fields_all_types = props.get_fields_all_types();
+        let fields_all_loaders: Vec<TokenStream2> = props
+            .get_fields_all()
+            .map(|field| {
+                let name = &field.ident;
+
+                let ty = field.get_type();
+
+                let mut converter = quote! {};
+
+                if let Some(_) = field.parse_custom_type() {
+                    let custom_ty = &field.ty;
+                    converter = quote! { <#custom_ty>::from };
+                }
+
+                quote! {
+                    #name: #converter(row.get::<&str, #ty>(concat!(stringify!(#name)))),
+                }
+            })
+            .collect();
+
+        let fields_ignored_names: Vec<&Option<syn::Ident>> = props
+            .get_ignored_fields()
+            .map(|field| &field.ident)
+            .collect();
+        let fields_ignored_types: Vec<&syn::Type> =
+            props.get_ignored_fields().map(|field| &field.ty).collect();
+
         quote! {
             fn from_row(row: &oxidizer::tokio_postgres::Row) -> Self {
                 let mut obj: Self = Self{
+                    #( #fields_all_loaders )*
                     #(
-                        #fields_all_names: row.get::<&str, #fields_all_types>(concat!(stringify!(#fields_all_names))),
+                        #fields_ignored_names: <#fields_ignored_types>::default(),
                     )*
                 };
                 obj
@@ -207,6 +251,17 @@ impl EntityBuilder {
         }
     }
 
+    fn build_is_synced_with_db_fn(&self, props: &Props) -> TokenStream2 {
+        let primary_key_ident = &props.get_primary_key_field().unwrap().ident;
+        let primary_key_type = &props.get_primary_key_field().unwrap().ty;
+        quote! {
+            fn is_synced_with_db(&self) -> bool {
+                let key_default: #primary_key_type = Default::default();
+                self.#primary_key_ident != key_default
+            }
+        }
+    }
+
     fn build_foreign_helpers(&self, props: &Props) -> Vec<TokenStream2> {
         let name = props.get_name();
 
@@ -215,6 +270,7 @@ impl EntityBuilder {
         foreign_fields.iter().map(|field| {
             let relation = field.parse_relation().unwrap();
             let local_key = field.ident.clone().unwrap();
+            let local_key_type = &field.ty;
             let get_ident = format_ident!("get_{}", to_snake_case(&relation.model));
             let set_ident = format_ident!("set_{}", to_snake_case(&relation.model));
             let trait_ident = format_ident!("__Accessor{}To{}", name, relation.model);
@@ -223,11 +279,7 @@ impl EntityBuilder {
 
             let local_key_set = match field.is_nullable() {
                 true => quote! {
-                    let key_default = Default::default();
-                    self.#local_key = match v.#key {
-                        vv if vv == key_default => None,
-                        vv => Some(vv),
-                    };
+                    self.#local_key = Some(v.#key);
                 },
                 false => quote! {
                     self.#local_key = v.#key;
@@ -244,7 +296,7 @@ impl EntityBuilder {
                 #[oxidizer::async_trait]
                 impl #trait_ident for #name {
                     async fn #get_ident(&self, db: &oxidizer::db::DB) -> oxidizer::db::DBResult<#model> {
-                        if self.#local_key == Default::default() {
+                        if self.#local_key == <#local_key_type>::default() {
                             return Err(oxidizer::db::Error::DoesNotExist);
                         }
 
@@ -259,7 +311,7 @@ impl EntityBuilder {
                     }
 
                     async fn #set_ident(&mut self, db: &oxidizer::db::DB, v: &#model) -> oxidizer::db::DBResult<()> {
-                        if v.#key == Default::default() {
+                        if !v.is_synced_with_db() {
                             return Err(oxidizer::db::Error::ReferencedModelIsNotInDB);
                         }
 
@@ -344,6 +396,7 @@ impl EntityBuilder {
 
         let save_fn = self.build_save_fn(&props);
         let delete_fn = self.build_delete_fn(&props);
+        let is_synced_with_db = self.build_is_synced_with_db_fn(&props);
         let from_row_fn = self.build_from_row_fn(&props);
         let create_migration_fn = self.build_create_migration_fn(&props);
         let find_fn = self.build_find_fn(&props);
@@ -363,6 +416,8 @@ impl EntityBuilder {
 
                 #delete_fn
 
+                #is_synced_with_db
+
                 #find_fn
 
                 #first_fn
